@@ -344,8 +344,10 @@ package_info::package_info ()
   name = NULL;
   installed_version = NULL;
   installed_section = NULL;
+  installed_pretty_name = NULL;
   available_version = NULL;
   available_section = NULL;
+  available_pretty_name = NULL;
   installed_short_description = NULL;
   available_short_description = NULL;
   installed_icon = NULL;
@@ -369,8 +371,10 @@ package_info::~package_info ()
   g_free (name);
   g_free (installed_version);
   g_free (installed_section);
+  g_free (installed_pretty_name);
   g_free (available_version);
   g_free (available_section);
+  g_free (available_pretty_name);
   g_free (installed_short_description);
   g_free (available_short_description);
   if (installed_icon)
@@ -388,6 +392,19 @@ package_info::~package_info ()
   g_free (dependencies);
 }
 
+const char *
+package_info::get_display_name (bool installed)
+{
+  const char *n;
+  if (installed || available_pretty_name == NULL)
+    n = installed_pretty_name;
+  else
+    n = available_pretty_name;
+  if (n == NULL)
+    n = name;
+  return n;
+}
+
 void
 package_info::ref ()
 {
@@ -576,12 +593,21 @@ compare_section_names (gconstpointer a, gconstpointer b)
 }
 
 static gint
-compare_package_names (gconstpointer a, gconstpointer b)
+compare_package_installed_names (gconstpointer a, gconstpointer b)
 {
-  package_info *pi_a = (package_info *)a;
-  package_info *pi_b = (package_info *)b;
+  const char *name_a = ((package_info *)a)->get_display_name (true);
+  const char *name_b = ((package_info *)b)->get_display_name (true);
+  
+  return package_sort_sign * g_ascii_strcasecmp (name_a, name_b);
+}
 
-  return package_sort_sign * g_ascii_strcasecmp (pi_a->name, pi_b->name);
+static gint
+compare_package_available_names (gconstpointer a, gconstpointer b)
+{
+  const char *name_a = ((package_info *)a)->get_display_name (false);
+  const char *name_b = ((package_info *)b)->get_display_name (false);
+  
+  return package_sort_sign * g_ascii_strcasecmp (name_a, name_b);
 }
 
 static gint
@@ -632,7 +658,7 @@ compare_package_download_sizes (gconstpointer a, gconstpointer b)
     return (package_sort_sign * 
 	    (pi_a->info.download_size - pi_b->info.download_size));
   else
-    return compare_package_names (a, b);
+    return compare_package_available_names (a, b);
 }
 
 void
@@ -651,8 +677,8 @@ sort_all_packages ()
 
   *section_ptr = g_list_sort (*section_ptr, compare_section_names);
 
-  GCompareFunc compare_packages_inst = compare_package_names;
-  GCompareFunc compare_packages_avail = compare_package_names;
+  GCompareFunc compare_packages_inst = compare_package_installed_names;
+  GCompareFunc compare_packages_avail = compare_package_available_names;
   if (package_sort_key == SORT_BY_VERSION)
     {
       compare_packages_inst = compare_package_installed_versions;
@@ -703,12 +729,14 @@ get_package_list_entry (apt_proto_decoder *dec)
   info->name = dec->decode_string_dup ();
   info->broken = dec->decode_int ();
   info->installed_version = dec->decode_string_dup ();
-  info->installed_size = dec->decode_int ();
+  info->installed_size = dec->decode_int64 ();
   info->installed_section = dec->decode_string_dup ();
+  info->installed_pretty_name = dec->decode_string_dup ();
   info->installed_short_description = dec->decode_string_dup ();
   installed_icon = dec->decode_string_in_place ();
   info->available_version = dec->decode_string_dup ();
   info->available_section = dec->decode_string_dup ();
+  info->available_pretty_name = dec->decode_string_dup ();
   info->available_short_description = dec->decode_string_dup ();
   available_icon = dec->decode_string_in_place ();
   
@@ -1630,14 +1658,14 @@ make_search_results_view (view *v)
 
 static void
 search_package_list (GList **result,
-		     GList *packages, const char *pattern)
+		     GList *packages, const char *pattern, bool installed)
 {
   while (packages)
     {
       package_info *pi = (package_info *)packages->data;
       
       // XXX
-      if (strcasestr (pi->name, pattern))
+      if (strcasestr (pi->get_display_name (installed), pattern))
 	{
 	  pi->ref ();
 	  *result = g_list_append (*result, pi);
@@ -1725,10 +1753,12 @@ search_packages_reply (int cmd, apt_proto_decoder *dec, void *data)
       dec->decode_int ();             // broken
       dec->decode_int ();             // installed_size
       dec->decode_string_in_place (); // installed_section
+      dec->decode_string_in_place (); // installed_pretty_name
       dec->decode_string_in_place (); // installed_short_description
       dec->decode_string_in_place (); // installed_icon
       dec->decode_string_in_place (); // available_version
       dec->decode_string_in_place (); // available_section
+      dec->decode_string_in_place (); // available_pretty_name
       dec->decode_string_in_place (); // available_short_description
       dec->decode_string_in_place (); // available_icon
 
@@ -1786,15 +1816,15 @@ search_packages (const char *pattern, bool in_descriptions)
 	  if (install_sections)
 	    {
 	      section_info *si = (section_info *)install_sections->data;
-	      search_package_list (&result, si->packages, pattern);
+	      search_package_list (&result, si->packages, pattern, false);
 	    }
 	}
       else if (parent == &upgrade_applications_view)
 	search_package_list (&result,
-			     upgradeable_packages, pattern);
+			     upgradeable_packages, pattern, false);
       else if (parent == &uninstall_applications_view)
 	search_package_list (&result,
-			     installed_packages, pattern);
+			     installed_packages, pattern, true);
 
       if (result)
 	{
@@ -1981,12 +2011,8 @@ restore_packages_flow ()
       
       if (backup)
 	{
-	  xexp *catalogues = xexp_aref (backup, "catalogues");
-	  if (catalogues)
-	    add_catalogues (catalogues, false, true,
-			    rp_restore, backup);
-	  else
-	    rp_restore (true, backup);
+	  refresh_package_cache_with_cont (APTSTATE_DEFAULT, false,
+					   rp_restore, backup);
 	}
       else
 	{
@@ -2000,27 +2026,22 @@ static void
 rp_restore (bool res, void *data)
 {
   xexp *backup = (xexp *)data;
-  xexp *packages = xexp_aref (backup, "packages");
-  if (packages)
-    {
-      int len = xexp_length (packages);
-      const char **names = (const char **)new char* [len+1];
 
-      xexp *p = xexp_first (packages);
-      int i = 0;
-      while (p)
-	{
-	  if (xexp_is (p, "pkg") && xexp_is_text (p))
-	    names[i++] = xexp_text (p);
-	  p = xexp_rest (p);
-	}
-      names[i] = NULL;
-      install_named_packages (APTSTATE_DEFAULT, names, INSTALL_TYPE_BACKUP,
-			      rp_end, backup);
-      delete names;
+  int len = xexp_length (backup);
+  const char **names = (const char **)new char* [len+1];
+
+  xexp *p = xexp_first (backup);
+  int i = 0;
+  while (p)
+    {
+      if (xexp_is (p, "pkg") && xexp_is_text (p))
+	names[i++] = xexp_text (p);
+      p = xexp_rest (p);
     }
-  else
-    rp_end (backup);
+  names[i] = NULL;
+  install_named_packages (APTSTATE_DEFAULT, names, INSTALL_TYPE_BACKUP,
+			  rp_end, backup);
+  delete names;
 }
 
 static void
@@ -2125,7 +2146,7 @@ mime_open_handler (gpointer raw_data, int argc, char **argv)
       const char *filename = argv[0];
 
       present_main_window ();
-      if (g_str_has_suffix (filename, ".backup"))
+      if (strcmp (filename, "backup.install") == 0)
 	restore_packages_flow ();
       else
 	install_from_file_flow (filename);
