@@ -25,6 +25,7 @@
 #include <assert.h>
 #include <iostream>
 #include <libintl.h>
+#include <errno.h>
 
 #include <gtk/gtk.h>
 #include <hildon/hildon-note.h>
@@ -108,7 +109,7 @@ installable_status_to_message (package_info *pi,
       msg = g_strdup_printf ((pi->installed_version
 			      ? _("ai_ni_error_update_failed")
 			      : _("ai_ni_error_installation_failed")),
-			     pi->name);
+			     pi->get_display_name (false));
       with_details = true;
     }
 
@@ -118,6 +119,10 @@ installable_status_to_message (package_info *pi,
 
 /* INSTALL_PACKAGES - Overview
 
+   XXX - system updates should be handled specially: they should be
+   installed first with their own special confirmation dialog, and
+   then the rest of the packages should be processed.
+
    0. Filter out already installed packages.  When the list is empty
       after this, an appropriate note is shown and the process aborts.
 
@@ -128,10 +133,12 @@ installable_status_to_message (package_info *pi,
 
    2. Make sure that the network is up when this is not a card install.
 
-   3. Check for certified packages.  For each of the selected
-      packages, a 'check_install' operation is performed and when one
-      of them would install non-certified packages, the Notice dialog
-      is shown.
+   3. Check for the trust status of all installed packages.  For each
+      of the selected packages, a 'check_install' operation is
+      performed and when one of them would install packages from a
+      non-trusted source, the Notice dialog is shown.  Also, when a
+      package would be upgraded from a different 'trust domain' than
+      it was originally installed from, the operation is aborted.
 
    The following is repeated for each selected package, as indicated.
    "Aborting this package" means that an error message is shown and
@@ -141,7 +148,7 @@ installable_status_to_message (package_info *pi,
    4. Check if the package is actually installable, and abort it when
       not.
 
-   XXX
+   xxx (download location)
    5. Check if enough storage is available and decide where to
       download the packages to.
 
@@ -155,20 +162,20 @@ installable_status_to_message (package_info *pi,
    XXX
    8. Check the free storage again.
 
-   xxx (close-apps flag)
+   xxx (actually closing all apps)
    9. If the package doesn't have the 'close-apps' flag, run the
-      'checkrm' scripts of the updated packages and abort this package
+      'checkrm' scripts of the upgraded packages and abort this package
       if the scripts asks for it.  Otherwise close all applications.
 
   10. Do the actual install, aborting this package if it fails.  The
       downloaded archive files are removed in any case.
 
-   XXX
-  11. If the package has the 'reboot' flag, reboot here and now.
+  11. If there are more packages to install, go back to 3.
 
-  12. If there are more packages to install, go back to 3.
+   At the end:
 
-   At the end, the following happens.
+   xxx (actually rebooting)
+  12. If any of the packages had the 'reboot' flag, reboot here and now.
 
   13. Refresh the lists of packages.
 */
@@ -181,9 +188,15 @@ struct ip_clos {
   GList *packages;       // the ones that are not installed or uptodate
   GList *cur;            // the one currently under consideration
 
+  // per installation iteration
+  int flags;
+  int64_t free_space;       // the required free storage space in bytes
   GSList *upgrade_names;    // the packages and versions that we are going
   GSList *upgrade_versions; // to upgrade to.
 
+  // at the end
+  bool reboot;              // whether to reboot
+  
   void (*cont) (void *);
   void *data;
 };
@@ -206,7 +219,7 @@ static void ip_check_upgrade_reply (int cmd, apt_proto_decoder *dec,
 				    void *data);
 static void ip_check_upgrade_loop (ip_clos *c);
 static void ip_check_upgrade_cmd_done (int status, void *data);
-static void ip_install_cur (ip_clos *c);
+static void ip_install_cur (void *data);
 static void ip_install_cur_reply (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_clean_reply (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_install_next (void *data);
@@ -242,10 +255,10 @@ xxx_install_packages (GList *packages,
   c->all_packages = packages;
   c->cont = cont;
   c->data = data;
-
   c->upgrade_names = NULL;
   c->upgrade_versions = NULL;
-
+  c->reboot = false;
+  
   // Filter packages, stopping after the first when this is a standard
   // install.
 
@@ -294,7 +307,8 @@ xxx_install_packages (GList *packages,
 		       (pi->installed_version
 			? _("ai_nc_update")
 			: _("ai_nc_install")),
-		       pi->name, pi->available_version, download_buf);
+		       pi->get_display_name (false),
+		       pi->available_version, download_buf);
   
       ask_yes_no_with_arbitrary_details ((pi->installed_version
 					  ? _("ai_ti_confirm_update")
@@ -399,10 +413,6 @@ ip_check_cert_loop (ip_clos *c)
 static void
 ip_check_cert_reply (int cmd, apt_proto_decoder *dec, void *data)
 {
-  /* XXX - we always show the "not-so-sure" variant of the legalese.
-           The 'trust-system' will be redone later.
-  */
-
   ip_clos *c = (ip_clos *)data;
 
   if (dec == NULL)
@@ -411,8 +421,41 @@ ip_check_cert_reply (int cmd, apt_proto_decoder *dec, void *data)
       return;
     }
 
-  apt_proto_preptype prep = apt_proto_preptype (dec->decode_int ());
-  if (prep != preptype_end)
+  /* XXX - any exceptional package causes us to show the 'not-so-sure'
+           version of the dialog.  This needs to be rethought once
+           apt-worker provides more details about the trust.
+  */
+
+  bool some_domains_violated = false;
+  bool some_not_certfied = false;
+
+  while (!dec->corrupted ())
+    {
+      apt_proto_pkgtrust trust = apt_proto_pkgtrust (dec->decode_int ());
+      if (trust == pkgtrust_end)
+	break;
+
+      if (trust == pkgtrust_not_certified)
+	some_not_certfied = true;
+      if (trust == pkgtrust_domains_violated)
+	some_domains_violated = true;
+
+      dec->decode_string_in_place ();  // name
+    }
+
+  // XXX - L10N
+
+  if (some_domains_violated)
+    {
+      if (red_pill_mode)
+	ask_custom (_("Trusted upgrade path broken"),
+		    "Continue anyway", "Stop",
+		    ip_legalese_response, c);
+      else
+	annoy_user_with_cont (_("Trusted upgrade path broken"),
+			      ip_end, c);
+    }
+  else if (some_not_certfied)
     scare_user_with_legalese (false, ip_legalese_response, c);
   else
     {
@@ -456,7 +499,6 @@ ip_install_loop (ip_clos *c)
   else
     {
       package_info *pi = (package_info *)(c->cur->data);
-      printf ("HAVE INFO %d, STATE %d\n", pi->have_info, c->state);
       get_intermediate_package_info (pi, true,
 				     ip_install_with_info, c, c->state);
     }
@@ -467,8 +509,6 @@ ip_install_with_info (package_info *pi, void *data, bool changed)
 {
   ip_clos *c = (ip_clos *)data;
   
-  printf ("STATUS: %d\n", pi->info.installable_status);
-
   if (pi->info.installable_status == status_able)
     {
       add_log ("-----\n");
@@ -477,11 +517,45 @@ ip_install_with_info (package_info *pi, void *data, bool changed)
 		 pi->installed_version, pi->available_version);
       else
 	add_log ("Installing %s %s\n", pi->name, pi->available_version);
-      
-      printf ("CHECK UPGRADE: %s\n", pi->name);
-      
-      apt_worker_install_check (c->state, pi->name,
-				ip_check_upgrade_reply, c);
+
+      int64_t free_space = get_free_space ();
+      if (free_space < 0)
+	annoy_user_with_errno (errno, "get_free_space",
+			       ip_end, c);
+
+      /* XXX - What is the right thing to do when a packages that asks
+    	       for a reboot fails during installation?  Is it better
+    	       to reboot or not?
+      */
+
+      if (pi->info.install_flags & pkgflag_reboot)
+	c->reboot = true;
+
+      if (pi->info.required_free_space + pi->info.download_size >= free_space)
+	{
+	  char free_string[20];
+	  char required_string[20];
+	  size_string_detailed (free_string, 20, free_space);
+	  size_string_detailed (required_string, 20,
+				pi->info.required_free_space
+				+ pi->info.download_size);
+
+	  char *msg = g_strdup_printf ("Not enough storage\n%s < %s",
+				       free_string, required_string);
+
+	  ip_abort_cur (c, msg, false);
+	}
+      else if (pi->info.install_flags & pkgflag_close_apps)
+	{
+	  // XXX - L10N
+	  annoy_user_with_cont (_("You should close all Applications now."),
+				ip_install_cur, c);
+	}
+      else
+	{
+	  apt_worker_install_check (c->state, pi->name,
+				    ip_check_upgrade_reply, c);
+	}
     }
   else
     ip_abort_cur_with_status_details (c);
@@ -506,8 +580,8 @@ ip_check_upgrade_reply (int cmd, apt_proto_decoder *dec, void *data)
 
   while (!dec->corrupted ())
     {
-      apt_proto_preptype prep = apt_proto_preptype (dec->decode_int ());
-      if (prep == preptype_end)
+      apt_proto_pkgtrust trust = apt_proto_pkgtrust (dec->decode_int ());
+      if (trust == pkgtrust_end)
 	break;
       
       dec->decode_string_in_place ();  // name
@@ -527,6 +601,12 @@ ip_check_upgrade_reply (int cmd, apt_proto_decoder *dec, void *data)
 
   int success = dec->decode_int ();
 
+  {
+    package_info *pi = (package_info *)(c->cur->data);
+    printf ("FLAGS: %x\n", pi->info.install_flags);
+    printf ("SPACE: %lu\n", pi->info.required_free_space);
+  }
+
   if (success)
     ip_check_upgrade_loop (c);
   else
@@ -591,8 +671,9 @@ ip_check_upgrade_cmd_done (int status, void *data)
 }
 
 static void
-ip_install_cur (ip_clos *c)
+ip_install_cur (void *data)
 {
+  ip_clos *c = (ip_clos *)data;
   package_info *pi = (package_info *)(c->cur->data);
 
   printf ("INSTALL %s\n", pi->name);
@@ -630,7 +711,7 @@ ip_install_cur_reply (int cmd, apt_proto_decoder *dec, void *data)
 	  char *str = g_strdup_printf ((pi->installed_version != NULL
 					? _("ai_ni_update_successful")
 					: _("ai_ni_install_successful")),
-				       pi->name);
+				       pi->get_display_name (false));
 	  annoy_user_with_cont (str, ip_install_next, c);
 	  g_free (str);
 	}
@@ -651,7 +732,7 @@ ip_install_cur_reply (int cmd, apt_proto_decoder *dec, void *data)
 	    msg = g_strdup_printf ((pi->installed_version != NULL
 				    ? _("ai_ni_error_update_failed")
 				    : _("ai_ni_error_installation_failed")),
-				   pi->name);
+				   pi->get_display_name (false));
 
 	  ip_abort_cur (c, msg, false);
 	  g_free (msg);
@@ -693,7 +774,6 @@ ip_abort_cur_with_status_details (ip_clos *c)
 static void
 ip_abort_cur (ip_clos *c, const char *msg, bool with_details)
 {
-  package_info *pi = (package_info *)(c->cur->data);
   bool is_last = (c->cur->next == NULL);
 
   GtkWidget *dialog;
@@ -777,7 +857,11 @@ ip_end (void *data)
   get_package_list (APTSTATE_DEFAULT);
   save_backup_data ();
 
-  c->cont (c->data);
+  if (c->reboot)
+    annoy_user_with_cont (_("You should reboot now"),
+			  c->cont, c->data);
+  else
+    c->cont (c->data);
   delete c;
 }
 
@@ -796,6 +880,7 @@ ip_end (void *data)
 struct up_clos {
   package_info *pi;
 
+  int flags;
   GSList *remove_names;
 
   void (*cont) (void *);
@@ -836,7 +921,8 @@ up_confirm (package_info *pi, void *data, bool changed)
   
   size_string_general (size_buf, 20, c->pi->installed_size);
   g_string_printf (text, _("ai_nc_uninstall"),
-		   c->pi->name, c->pi->installed_version, size_buf);
+		   c->pi->get_display_name (true),
+		   c->pi->installed_version, size_buf);
 
   ask_yes_no_with_details (_("ai_ti_confirm_uninstall"), text->str,
 			   c->pi, remove_details,
@@ -850,8 +936,8 @@ up_checkrm_start (bool res, void *data)
   up_clos *c = (up_clos *)data;
 
   if (res)
-    apt_worker_get_packages_to_remove (c->pi->name,
-				       up_checkrm_reply, c);
+    apt_worker_remove_check (c->pi->name,
+			     up_checkrm_reply, c);
   else
     up_end (c);
 }
@@ -909,7 +995,7 @@ up_checkrm_cmd_done (int status, void *data)
 
       char *str =
 	g_strdup_printf (_("ai_ni_error_uninstall_applicationrunning"),
-			 c->pi->name);
+			 c->pi->get_display_name (true));
       annoy_user_with_cont (str, up_end, c);
       g_free (str);
     }
@@ -930,6 +1016,9 @@ up_remove (up_clos *c)
     }
   else
     {
+      if (c->pi->info.removable_status == status_system_update_unremovable)
+	annoy_user_with_cont
+	  (_("ai_ni_error_system"), up_end, c);
       if (c->pi->info.removable_status == status_needed)
 	annoy_user_with_details_with_cont
 	  (_("ai_ni_error_uninstall_packagesneeded"),
@@ -937,7 +1026,7 @@ up_remove (up_clos *c)
       else
 	{
 	  char *str = g_strdup_printf (_("ai_ni_error_uninstallation_failed"),
-				       c->pi->name);
+				       c->pi->get_display_name (true));
 	  annoy_user_with_details_with_cont (str, c->pi, remove_details,
 					     up_end, c);
 	  g_free (str);
@@ -965,14 +1054,14 @@ up_remove_reply (int cmd, apt_proto_decoder *dec, void *data)
   if (success)
     {
       char *str = g_strdup_printf (_("ai_ni_uninstall_successful"),
-				   c->pi->name);
+				   c->pi->get_display_name (true));
       annoy_user_with_cont (str, up_end, c);
       g_free (str);
     }
   else
     {
       char *str = g_strdup_printf (_("ai_ni_error_uninstallation_failed"),
-				   c->pi->name);
+				   c->pi->get_display_name (true));
       annoy_user_with_cont (str, up_end, c);
       g_free (str);
     }
@@ -1065,6 +1154,7 @@ if_details_reply (int cmd, apt_proto_decoder *dec, void *data)
   c->pi = pi;
 
   pi->name = dec->decode_string_dup ();
+  pi->available_pretty_name = dec->decode_string_dup ();
   pi->broken = false;
   pi->installed_version = dec->decode_string_dup ();
   pi->installed_size = dec->decode_int ();;
@@ -1099,10 +1189,12 @@ if_details_reply (int cmd, apt_proto_decoder *dec, void *data)
   size_string_general (size_buf, 20, pi->info.install_user_size_delta);
   if (pi->installed_version)
     g_string_printf (text, _("ai_nc_update"),
-		     pi->name, pi->available_version, size_buf);
+		     pi->get_display_name (false),
+		     pi->available_version, size_buf);
   else
     g_string_printf (text, _("ai_nc_install"),
-		     pi->name, pi->available_version, size_buf);
+		     pi->get_display_name (false),
+		     pi->available_version, size_buf);
 
   void (*cont) (bool res, void *);
 
@@ -1172,7 +1264,7 @@ if_install_reply (int cmd, apt_proto_decoder *dec, void *data)
       char *str = g_strdup_printf (c->pi->installed_version
 				   ? _("ai_ni_update_successful")
 				   : _("ai_ni_install_successful"),
-				   c->pi->name);
+				   c->pi->get_display_name (false));
       annoy_user_with_cont (str, if_end, c);
       g_free (str);
     }
@@ -1187,7 +1279,7 @@ if_install_reply (int cmd, apt_proto_decoder *dec, void *data)
 	msg = g_strdup_printf (c->pi->installed_version
 			       ? _("ai_ni_error_update_failed")
 			       : _("ai_ni_error_installation_failed"),
-			       c->pi->name);
+			       c->pi->get_display_name (false));
 
       annoy_user_with_cont (msg, if_end, c);
       g_free (msg);
