@@ -216,6 +216,22 @@ xexp_del (xexp *x, xexp *z)
   g_return_if_reached ();
 }
 
+xexp *
+xexp_pop (xexp *x)
+{
+  xexp *y;
+
+  g_return_val_if_fail (xexp_is_list (x), NULL);
+
+  y = x->first;
+  if (y)
+    {
+      x->first = y->rest;
+      y->rest = NULL;
+    }
+  return y;
+}
+
 xexp *
 xexp_text_new (const char *tag, const char *text)
 {
